@@ -1,7 +1,7 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2020 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2022 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -24,8 +24,17 @@
 #include <Python.h>
 
 /* Sanity check on the Python version. */
-#if PY_VERSION_HEX < 0x03050000
-#error "This version of SIP requires Python v3.5 or later"
+#if PY_VERSION_HEX < 0x03070000
+#error "This version of PyQt5.sip requires Python v3.7 or later"
+#endif
+
+
+#ifdef __cplusplus
+#include <exception>
+
+typedef bool (*sipExceptionHandler)(std::exception_ptr);
+#else
+typedef void *sipExceptionHandler;
 #endif
 
 
@@ -34,26 +43,35 @@ extern "C" {
 #endif
 
 
-/* The patch version of this implementation of the ABI. */
-#define SIP_MODULE_PATCH_VERSION    2
+/* The version of the ABI. */
+#define SIP_ABI_MAJOR_VERSION       12
+#define SIP_ABI_MINOR_VERSION       11
+#define SIP_MODULE_PATCH_VERSION    0
 
 
 /*
- * The changes to this version of the ABI.
+ * The change history of the ABI.
  *
- * Added sip_api_visit_wrappers() to the public API.
- * Added sip_api_register_exit_notifier() to the public API.
- * sip_api_is_owned_by_python() is now part of the public API.
+ * v12.11
+ *  - Published the 'array' type.
+ *  - Added the ctd_sizeof, ctd_array_delete members to sipClassTypeDef.
+ *  - Added the '>' conversion character to sipParseArgs().
+ *
+ * v12.10
+ *  - Python v3.7 or later is required.
+ *  - Added support for Python v3.11.
+ *
+ * v12.9
+ *  - Added sipNextExceptionHandler().
+ *
+ * v12.8
+ *  - Added a new thread-safe implementation of sipIsPyMethod().
  */
 
 
-/* The ABI version implemented. */
-#define SIP_ABI_MAJOR_VERSION       12
-#define SIP_ABI_MINOR_VERSION       7
-
 /* The version of the code generator. */
-#define SIP_VERSION                 0x50102
-#define SIP_VERSION_STR             "5.1.2"
+#define SIP_VERSION                 0x60602
+#define SIP_VERSION_STR             "6.6.2"
 
 /* These are all dependent on the user-specified name of the sip module. */
 #define _SIP_MODULE_FQ_NAME         "PyQt5.sip"
@@ -112,16 +130,11 @@ typedef unsigned int uint;
 #endif
 
 
-/* Remove in v5.1. */
 #define SIP_SSIZE_T             Py_ssize_t
 #define SIP_SSIZE_T_FORMAT      "%zd"
 #define SIP_USE_PYCAPSULE
 #define SIP_MODULE_RETURN(v)    return (v)
 
-/*
- * Remove in v5.1.  These are undocumented and can be removed when PyQt5 drops
- * support for Python v2.
- */
 #define SIPLong_Check       PyLong_Check
 #define SIPLong_FromLong    PyLong_FromLong
 #define SIPLong_AsLong      PyLong_AsLong
@@ -252,6 +265,7 @@ typedef int (*sipVirtHandlerFunc)(sip_gilstate_t, sipVirtErrorHandlerFunc,
         sipSimpleWrapper *, PyObject *, ...);
 typedef void (*sipAssignFunc)(void *, Py_ssize_t, void *);
 typedef void *(*sipArrayFunc)(Py_ssize_t);
+typedef void (*sipArrayDeleteFunc)(void *);
 typedef void *(*sipCopyFunc)(const void *, Py_ssize_t);
 typedef void (*sipReleaseFunc)(void *, int);
 typedef PyObject *(*sipPickleFunc)(void *);
@@ -357,7 +371,6 @@ struct _sipWrapper {
 
 
 /*
- * Removed in v5.1.
  * The meta-type of an enum type.  (This is exposed only to support the
  * deprecated sipConvertFromNamedEnum() macro.)
  */
@@ -880,6 +893,12 @@ typedef struct _sipClassTypeDef {
 
     /* The mixin initialisation function. */
     initproc ctd_init_mixin;
+
+    /* The optional array delete function. */
+    sipArrayDeleteFunc ctd_array_delete;
+
+    /* The sizeof the class. */
+    size_t ctd_sizeof;
 } sipClassTypeDef;
 
 
@@ -944,7 +963,6 @@ typedef struct _sipExternalTypeDef {
 
 
 /*
- * Remove in v5.1.
  * The information describing a mapped class.  This (and anything that uses it)
  * is deprecated.
  */
@@ -1142,6 +1160,9 @@ typedef struct _sipExportedModuleDef {
 
     /* The optional table of versioned functions. */
     sipVersionedFunctionDef *em_versioned_functions;
+
+    /* The exception handler. */
+    sipExceptionHandler em_exception_handler;
 } sipExportedModuleDef;
 
 
@@ -1313,7 +1334,6 @@ typedef struct _sipTypeInstanceDef {
 
 
 /*
- * Remove in v5.1.
  * Define a mapping between a wrapped type identified by a string and the
  * corresponding Python type.
  */
@@ -1327,7 +1347,6 @@ typedef struct _sipStringTypeClassMap {
 
 
 /*
- * Remove in v5.1.
  * Define a mapping between a wrapped type identified by an integer and the
  * corresponding Python type.
  */
@@ -1639,6 +1658,13 @@ typedef struct _sipAPIDef {
     size_t (*api_long_as_size_t)(PyObject *o);
     void (*api_visit_wrappers)(sipWrapperVisitorFunc visitor, void *closure);
     int (*api_register_exit_notifier)(PyMethodDef *md);
+
+    /*
+     * The following are not part of the public API.
+     */
+    PyObject *(*api_is_py_method_12_8)(sip_gilstate_t *gil, char *pymc,
+            sipSimpleWrapper **sipSelfp, const char *cname, const char *mname);
+    sipExceptionHandler (*api_next_exception_handler)(void **statep);
 } sipAPIDef;
 
 const sipAPIDef *sip_init_library(PyObject *mod_dict);
@@ -1759,9 +1785,8 @@ typedef struct _sipQtAPI {
 #define sipTypeName(td)     sipNameFromPool((td)->td_module, (td)->td_cname)
 #define sipTypePluginData(td)   ((td)->td_plugin_data)
 
-
 /*
- * Remove in v5.1.
+ * These are deprecated.
  */
 #define sipClassName(w)     PyString_FromString(Py_TYPE(w)->tp_name)
 #define sipIsExactWrappedType(wt)   (sipTypeAsPyTypeObject((wt)->wt_td) == (PyTypeObject *)(wt))
